'use strict';

exports.__esModule = true;
exports.default = createObservableState;

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _mobx = require('mobx');

var _parser = require('../parser');

var _validateContent = require('../ingredients/validateContent');

var _validateContent2 = _interopRequireDefault(_validateContent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var suggestReferenceFromTree = _ramda2.default.uncurryN(2, function (ingredients) {
	return _ramda2.default.pipe(_ramda2.default.chain(_ramda2.default.pluck('references')), _ramda2.default.unnest, _ramda2.default.map(_ramda2.default.head), // Just the requested ingredient ID
	_ramda2.default.difference(_ramda2.default.__, _ramda2.default.pluck('id', ingredients)), // Only IDs that are yet to be used
	_ramda2.default.head // First pick
	);
});

function createObservableIngredientVariation(ingredient, _ref) {
	var _ref$enabled = _ref.enabled,
	    enabled = _ref$enabled === undefined ? true : _ref$enabled,
	    rawContent = _ref.rawContent;

	return (0, _mobx.observable)({
		enabled: enabled,
		rawContent: rawContent,
		get result() {
			var transform = (0, _validateContent.transformerForType)(ingredient.type);
			return _ramda2.default.tryCatch(transform, _ramda2.default.objOf('error'))(this.rawContent);
		},
		toggleEnabled: (0, _mobx.action)(function () {
			this.enabled = !this.enabled;
		}),
		adjustRawContent: (0, _mobx.action)(function (adjuster) {
			this.rawContent = adjuster(this.rawContent);
		}),
		adjustContent: (0, _mobx.action)(function (adjuster) {
			var result = this.result;
			if (result.content) {
				adjuster(result.content);
				this.rawContent = (0, _validateContent.stringRepresenterForType)(ingredient.type, result.content);
			}
		}),
		editPath: (0, _mobx.action)(function (path, editor) {
			this.adjustContent(function (content) {
				var _ref2 = path.length == 1 ? [content, path[0]] : [_ramda2.default.path(initialPath, content), _ramda2.default.last(path)],
				    parent = _ref2[0],
				    key = _ref2[1];

				if (parent) {
					editor(parent, key);
				} else {
					// Warn user about incorrect key path?
				}
			});
		}),
		adjustPath: (0, _mobx.action)(function (path, adjuster) {
			this.editPath(path, function (parent, key) {
				parent[key] = adjuster(parent[key]);
			});
		})
	});
}

function createObservableIngredient(target) {
	return (0, _mobx.extendObservable)((0, _mobx.observable)(target), {
		variations: target.variations.map(_ramda2.default.curry(createObservableIngredientVariation)(target)),
		addVariation: (0, _mobx.action)(function (variation) {
			target.variations.push(createObservableIngredientVariation(target, variation));
		}),
		addNewVariation: (0, _mobx.action)(function (variation) {
			target.addVariation({
				rawContent: ''
			});
		}),
		get flattenedResult() {
			var flattened = {
				type: target.type,
				variationReference: _ramda2.default.last(target.variations)
			};

			if (target.type == 'json') {
				flattened.content = target.variations.reduce(function (combined, _ref3) {
					var enabled = _ref3.enabled,
					    result = _ref3.result;

					if (!enabled || result.content == null) {
						return combined;
					}

					return Object.assign(combined, result.content);
				}, {});
			} else {
				var variation = _ramda2.default.findLast(_ramda2.default.propEq('enabled', true), target.variations);
				if (variation) {
					flattened.content = variation.result.content;
				}
			}

			return flattened;
		}
	});
}

function createObservableState(_ref4) {
	var content = _ref4.content,
	    allIngredients = _ref4.allIngredients,
	    scenarios = _ref4.scenarios,
	    activeScenarioIndex = _ref4.activeScenarioIndex,
	    destinationID = _ref4.destinationID,
	    destinationDevice = _ref4.destinationDevice;

	var target = this;
	return (0, _mobx.extendObservable)(target, {
		content: content,
		get contentTree() {
			return (0, _parser.parseInput)(this.content);
		},
		allIngredients: allIngredients.map(createObservableIngredient),
		scenarios: scenarios,
		activeScenarioIndex: activeScenarioIndex,
		destinationID: destinationID,
		destinationDevice: destinationDevice,
		get activeScenario() {
			return this.scenarios[this.activeScenarioIndex];
		},
		get activeIngredients() {
			var activeScenario = this.activeScenario;
			return this.allIngredients.reduce(function (object, _ref5) {
				var id = _ref5.id,
				    flattenedResult = _ref5.flattenedResult;

				object[id] = flattenedResult;
				return object;
			}, {});
		},
		activeVariationForIngredientAtIndex: function activeVariationForIngredientAtIndex(index) {
			var ingredient = this.allIngredients[index];
			if (ingredient && ingredient.variations.length > 0) {
				var activeScenario = this.activeScenario;
				return ingredient.variations[_ramda2.default.propOr(0, ingredient.id, activeScenario)];
			}
		},

		addNewIngredient: (0, _mobx.action)(function () {
			target.allIngredients.push(createObservableIngredient({
				id: _ramda2.default.defaultTo('untitled', suggestReferenceFromTree(target.allIngredients, target.contentTree)),
				type: 'text',
				variations: [{
					rawContent: ''
				}]
			}));
		}),
		// Use target to allow prebinding
		onRemoveIngredientAtIndex: (0, _mobx.action)(function (index) {
			target.allIngredients.splice(index, 1);
		})
	});
}
module.exports = exports['default'];